@@ -1,13 +1,15 @@
-# Copyright (C) 2015-2019  The Software Heritage developers
+# Copyright (C) 2015-2020  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import random
 import select
 
 from swh.core.db import BaseDb
 from swh.core.db.db_utils import stored_procedure, jsonize
 from swh.core.db.db_utils import execute_values_generator
+from swh.model.model import SHA1_SIZE
 
 
 class Db(BaseDb):
@@ -72,11 +74,10 @@ class Db(BaseDb):
 
     def content_get_metadata_from_sha1s(self, sha1s, cur=None):
         cur = self._cursor(cur)
-
         yield from execute_values_generator(
             cur, """
             select t.sha1, %s from (values %%s) as t (sha1)
-            left join content using (sha1)
+            inner join content using (sha1)
             """ % ', '.join(self.content_get_metadata_keys[1:]),
             ((sha1,) for sha1 in sha1s),
         )
@@ -125,6 +126,15 @@ class Db(BaseDb):
             SELECT 1 FROM content c WHERE c.sha1 = t.sha1
         )""", ((sha1,) for sha1 in sha1s))
 
+    def content_missing_per_sha1_git(self, contents, cur=None):
+        cur = self._cursor(cur)
+
+        yield from execute_values_generator(cur, """
+        SELECT t.sha1_git FROM (VALUES %s) AS t(sha1_git)
+        WHERE NOT EXISTS (
+            SELECT 1 FROM content c WHERE c.sha1_git = t.sha1_git
+        )""", ((sha1,) for sha1 in contents))
+
     def skipped_content_missing(self, contents, cur=None):
         if not contents:
             return []
@@ -133,9 +143,9 @@ class Db(BaseDb):
         query = """SELECT * FROM (VALUES %s) AS t (%s)
                    WHERE not exists
                    (SELECT 1 FROM skipped_content s WHERE
-                       s.sha1 is not distinct from t.sha1 and
-                       s.sha1_git is not distinct from t.sha1_git and
-                       s.sha256 is not distinct from t.sha256);""" % \
+                       s.sha1 is not distinct from t.sha1::sha1 and
+                       s.sha1_git is not distinct from t.sha1_git::sha1 and
+                       s.sha256 is not distinct from t.sha256::bytea);""" % \
                 ((', '.join('%s' for _ in contents)),
                  ', '.join(self.content_hash_keys))
         cur.execute(query,
@@ -152,6 +162,16 @@ class Db(BaseDb):
 
         return bool(cur.fetchone())
 
+    def snapshot_missing_from_list(self, snapshots, cur=None):
+        cur = self._cursor(cur)
+        yield from execute_values_generator(
+            cur, """
+            SELECT id FROM (VALUES %s) as t(id)
+            WHERE NOT EXISTS (
+                SELECT 1 FROM snapshot d WHERE d.id = t.id
+            )
+                """, ((id,) for id in snapshots))
+
     def snapshot_add(self, snapshot_id, cur=None):
         """Add a snapshot from the temporary table"""
         cur = self._cursor(cur)
@@ -186,18 +206,23 @@ class Db(BaseDb):
 
         yield from cur
 
-    def snapshot_get_by_origin_visit(self, origin_id, visit_id, cur=None):
+    def snapshot_get_by_origin_visit(self, origin_url, visit_id, cur=None):
         cur = self._cursor(cur)
         query = """\
-           SELECT snapshot from origin_visit where
-           origin_visit.origin=%s and origin_visit.visit=%s;
+           SELECT snapshot FROM origin_visit
+           INNER JOIN origin ON origin.id = origin_visit.origin
+           WHERE origin.url=%s AND origin_visit.visit=%s;
         """
 
-        cur.execute(query, (origin_id, visit_id))
+        cur.execute(query, (origin_url, visit_id))
         ret = cur.fetchone()
         if ret:
             return ret[0]
 
+    def snapshot_get_random(self, cur=None):
+        return self._get_random_row_from_table(
+            'snapshot', ['id'], 'id', cur)
+
     content_find_cols = ['sha1', 'sha1_git', 'sha256', 'blake2s256', 'length',
                          'ctime', 'status']
 
@@ -236,6 +261,10 @@ class Db(BaseDb):
         content = cur.fetchall()
         return content
 
+    def content_get_random(self, cur=None):
+        return self._get_random_row_from_table(
+            'content', ['sha1_git'], 'sha1_git', cur)
+
     def directory_missing_from_list(self, directories, cur=None):
         cur = self._cursor(cur)
         yield from execute_values_generator(
@@ -279,6 +308,10 @@ class Db(BaseDb):
             return None
         return data
 
+    def directory_get_random(self, cur=None):
+        return self._get_random_row_from_table(
+            'directory', ['id'], 'id', cur)
+
     def revision_missing_from_list(self, revisions, cur=None):
         cur = self._cursor(cur)
 
@@ -298,8 +331,7 @@ class Db(BaseDb):
         'committer_email', 'metadata', 'synthetic',
     ]
 
-    revision_get_cols = revision_add_cols + [
-        'author_id', 'committer_id', 'parents']
+    revision_get_cols = revision_add_cols + ['parents']
 
     def origin_visit_add(self, origin, ts, type, cur=None):
         """Add a new origin_visit for origin origin at timestamp ts with
@@ -324,9 +356,10 @@ class Db(BaseDb):
         cur = self._cursor(cur)
         update_cols = []
         values = []
-        where = ['origin=%s AND visit=%s']
+        where = ['origin.id = origin_visit.origin',
+                 'origin.url=%s',
+                 'visit=%s']
         where_values = [origin_id, visit_id]
-        from_ = ''
         if 'status' in updates:
             update_cols.append('status=%s')
             values.append(updates.pop('status'))
@@ -338,17 +371,20 @@ class Db(BaseDb):
             values.append(updates.pop('snapshot'))
         assert not updates, 'Unknown fields: %r' % updates
         query = """UPDATE origin_visit
-                    SET {update_cols}
-                    {from}
-                    WHERE {where}""".format(**{
+                   SET {update_cols}
+                   FROM origin
+                   WHERE {where}""".format(**{
             'update_cols': ', '.join(update_cols),
-            'from': from_,
             'where': ' AND '.join(where)
         })
         cur.execute(query, (*values, *where_values))
 
     def origin_visit_upsert(self, origin, visit, date, type, status,
                             metadata, snapshot, cur=None):
+        # doing an extra query like this is way simpler than trying to join
+        # the origin id in the query below
+        origin_id = next(self.origin_id_get_by_url([origin]))
+
         cur = self._cursor(cur)
         query = """INSERT INTO origin_visit ({cols}) VALUES ({values})
                    ON CONFLICT ON CONSTRAINT origin_visit_pkey DO
@@ -358,10 +394,14 @@ class Db(BaseDb):
                 updates=', '.join('{0}=excluded.{0}'.format(col)
                                   for col in self.origin_visit_get_cols))
         cur.execute(
-            query, (origin, visit, date, type, status, metadata, snapshot))
+            query, (origin_id, visit, date, type, status, metadata, snapshot))
 
-    origin_visit_get_cols = ['origin', 'visit', 'date', 'type', 'status',
-                             'metadata', 'snapshot']
+    origin_visit_get_cols = [
+        'origin', 'visit', 'date', 'type',
+        'status', 'metadata', 'snapshot']
+    origin_visit_select_cols = [
+        'origin.url AS origin', 'visit', 'date', 'origin_visit.type AS type',
+        'status', 'metadata', 'snapshot']
 
     def origin_visit_get_all(self, origin_id,
                              last_visit=None, limit=None, cur=None):
@@ -386,10 +426,11 @@ class Db(BaseDb):
         query = """\
         SELECT %s
         FROM origin_visit
-        WHERE origin=%%s %s
+        INNER JOIN origin ON origin.id = origin_visit.origin
+        WHERE origin.url=%%s %s
         order by visit asc
         limit %%s""" % (
-            ', '.join(self.origin_visit_get_cols), extra_condition
+            ', '.join(self.origin_visit_select_cols), extra_condition
         )
 
         cur.execute(query, args)
@@ -412,8 +453,9 @@ class Db(BaseDb):
         query = """\
             SELECT %s
             FROM origin_visit
-            WHERE origin = %%s AND visit = %%s
-            """ % (', '.join(self.origin_visit_get_cols))
+            INNER JOIN origin ON origin.id = origin_visit.origin
+            WHERE origin.url = %%s AND visit = %%s
+            """ % (', '.join(self.origin_visit_select_cols))
 
         cur.execute(query, (origin_id, visit_id))
         r = cur.fetchall()
@@ -458,10 +500,11 @@ class Db(BaseDb):
         cur = self._cursor(cur)
 
         query_parts = [
-            'SELECT %s' % ', '.join(self.origin_visit_get_cols),
-            'FROM origin_visit']
+            'SELECT %s' % ', '.join(self.origin_visit_select_cols),
+            'FROM origin_visit',
+            'INNER JOIN origin ON origin.id = origin_visit.origin']
 
-        query_parts.append('WHERE origin = %s')
+        query_parts.append('WHERE origin.url = %s')
 
         if require_snapshot:
             query_parts.append('AND snapshot is not null')
@@ -481,6 +524,30 @@ class Db(BaseDb):
             return None
         return r
 
+    def origin_visit_get_random(self, type, cur=None):
+        """Randomly select one origin visit that was full and in the last 3
+           months
+
+        """
+        cur = self._cursor(cur)
+        columns = ','.join(self.origin_visit_select_cols)
+        query = f"""with visits as (
+                      select *
+                      from origin_visit
+                      where origin_visit.status='full' and
+                            origin_visit.type=%s and
+                            origin_visit.date > now() - '3 months'::interval
+                    )
+                    select {columns}
+                    from visits as origin_visit
+                    inner join origin
+                    on origin_visit.origin=origin.id
+                    where random() < 0.1
+                    limit 1
+                 """
+        cur.execute(query, (type, ))
+        return cur.fetchone()
+
     @staticmethod
     def mangle_query_key(key, main_table):
         if key == 'id':
@@ -542,6 +609,10 @@ class Db(BaseDb):
         cur.execute(query, (root_revisions, limit))
         yield from cur
 
+    def revision_get_random(self, cur=None):
+        return self._get_random_row_from_table(
+            'revision', ['id'], 'id', cur)
+
     def release_missing_from_list(self, releases, cur=None):
         cur = self._cursor(cur)
         yield from execute_values_generator(
@@ -552,50 +623,46 @@ class Db(BaseDb):
             )
             """, ((id,) for id in releases))
 
-    object_find_by_sha1_git_cols = ['sha1_git', 'type', 'id', 'object_id']
+    object_find_by_sha1_git_cols = ['sha1_git', 'type']
 
     def object_find_by_sha1_git(self, ids, cur=None):
         cur = self._cursor(cur)
 
         yield from execute_values_generator(
             cur, """
-            WITH t (id) AS (VALUES %s),
+            WITH t (sha1_git) AS (VALUES %s),
             known_objects as ((
                 select
                   id as sha1_git,
                   'release'::object_type as type,
-                  id,
                   object_id
                 from release r
-                where exists (select 1 from t where t.id = r.id)
+                where exists (select 1 from t where t.sha1_git = r.id)
             ) union all (
                 select
                   id as sha1_git,
                   'revision'::object_type as type,
-                  id,
                   object_id
                 from revision r
-                where exists (select 1 from t where t.id = r.id)
+                where exists (select 1 from t where t.sha1_git = r.id)
             ) union all (
                 select
                   id as sha1_git,
                   'directory'::object_type as type,
-                  id,
                   object_id
                 from directory d
-                where exists (select 1 from t where t.id = d.id)
+                where exists (select 1 from t where t.sha1_git = d.id)
             ) union all (
                 select
                   sha1_git as sha1_git,
                   'content'::object_type as type,
-                  sha1 as id,
                   object_id
                 from content c
-                where exists (select 1 from t where t.id = c.sha1_git)
+                where exists (select 1 from t where t.sha1_git = c.sha1_git)
             ))
-            select t.id as sha1_git, k.type, k.id, k.object_id
+            select t.sha1_git as sha1_git, k.type
             from t
-            left join known_objects k on t.id = k.sha1_git
+            left join known_objects k on t.sha1_git = k.sha1_git
             """,
             ((id,) for id in ids)
         )
@@ -605,87 +672,51 @@ class Db(BaseDb):
         cur.execute('SELECT * FROM swh_stat_counters()')
         yield from cur
 
-    fetch_history_cols = ['origin', 'date', 'status', 'result', 'stdout',
-                          'stderr', 'duration']
-
-    def create_fetch_history(self, fetch_history, cur=None):
-        """Create a fetch_history entry with the data in fetch_history"""
-        cur = self._cursor(cur)
-        query = '''INSERT INTO fetch_history (%s)
-                   VALUES (%s) RETURNING id''' % (
-            ','.join(self.fetch_history_cols),
-            ','.join(['%s'] * len(self.fetch_history_cols))
-        )
-        cur.execute(query, [fetch_history.get(col) for col in
-                            self.fetch_history_cols])
-
-        return cur.fetchone()[0]
-
-    def get_fetch_history(self, fetch_history_id, cur=None):
-        """Get a fetch_history entry with the given id"""
-        cur = self._cursor(cur)
-        query = '''SELECT %s FROM fetch_history WHERE id=%%s''' % (
-            ', '.join(self.fetch_history_cols),
-        )
-        cur.execute(query, (fetch_history_id,))
-
-        data = cur.fetchone()
-
-        if not data:
-            return None
-
-        ret = {'id': fetch_history_id}
-        for i, col in enumerate(self.fetch_history_cols):
-            ret[col] = data[i]
-
-        return ret
-
-    def update_fetch_history(self, fetch_history, cur=None):
-        """Update the fetch_history entry from the data in fetch_history"""
-        cur = self._cursor(cur)
-        query = '''UPDATE fetch_history
-                   SET %s
-                   WHERE id=%%s''' % (
-            ','.join('%s=%%s' % col for col in self.fetch_history_cols)
-        )
-        cur.execute(query, [jsonize(fetch_history.get(col)) for col in
-                            self.fetch_history_cols + ['id']])
-
-    def origin_add(self, type, url, cur=None):
+    def origin_add(self, url, cur=None):
         """Insert a new origin and return the new identifier."""
-        insert = """INSERT INTO origin (type, url) values (%s, %s)
-                    RETURNING id"""
+        insert = """INSERT INTO origin (url) values (%s)
+                    RETURNING url"""
 
-        cur.execute(insert, (type, url))
+        cur.execute(insert, (url,))
         return cur.fetchone()[0]
 
-    origin_cols = ['id', 'type', 'url']
+    origin_cols = ['url']
 
-    def origin_get_with(self, origins, cur=None):
-        """Retrieve the origin id from its type and url if found."""
+    def origin_get_by_url(self, origins, cur=None):
+        """Retrieve origin `(type, url)` from urls if found."""
         cur = self._cursor(cur)
 
-        query = """SELECT %s FROM (VALUES %%s) as t(type, url)
-                   LEFT JOIN origin
-                       ON ((t.type IS NULL OR t.type=origin.type)
-                           AND t.url=origin.url)
+        query = """SELECT %s FROM (VALUES %%s) as t(url)
+                   LEFT JOIN origin ON t.url = origin.url
                 """ % ','.join('origin.' + col for col in self.origin_cols)
 
         yield from execute_values_generator(
-            cur, query, origins)
+            cur, query, ((url,) for url in origins))
 
-    def origin_get(self, ids, cur=None):
-        """Retrieve the origin per its identifier.
-
-        """
+    def origin_get_by_sha1(self, sha1s, cur=None):
+        """Retrieve origin urls from sha1s if found."""
         cur = self._cursor(cur)
 
-        query = """SELECT %s FROM (VALUES %%s) as t(id)
-                   LEFT JOIN origin ON t.id = origin.id
+        query = """SELECT %s FROM (VALUES %%s) as t(sha1)
+                   LEFT JOIN origin ON t.sha1 = digest(origin.url, 'sha1')
                 """ % ','.join('origin.' + col for col in self.origin_cols)
 
         yield from execute_values_generator(
-            cur, query, ((id,) for id in ids))
+            cur, query, ((sha1,) for sha1 in sha1s))
+
+    def origin_id_get_by_url(self, origins, cur=None):
+        """Retrieve origin `(type, url)` from urls if found."""
+        cur = self._cursor(cur)
+
+        query = """SELECT id FROM (VALUES %s) as t(url)
+                   LEFT JOIN origin ON t.url = origin.url
+                """
+
+        for row in execute_values_generator(
+                cur, query, ((url,) for url in origins)):
+            yield row[0]
+
+    origin_get_range_cols = ['id', 'url']
 
     def origin_get_range(self, origin_from=1, origin_count=100, cur=None):
         """Retrieve ``origin_count`` origins whose ids are greater
@@ -702,7 +733,7 @@ class Db(BaseDb):
         query = """SELECT %s
                    FROM origin WHERE id >= %%s
                    ORDER BY id LIMIT %%s
-                """ % ','.join(self.origin_cols)
+                """ % ','.join(self.origin_get_range_cols)
 
         cur.execute(query, (origin_from, origin_count))
         yield from cur
@@ -724,7 +755,12 @@ class Db(BaseDb):
                    WHERE """
         if with_visit:
             query += """
-                   EXISTS (SELECT 1 from origin_visit WHERE origin=origin.id)
+                   EXISTS (
+                     SELECT 1
+                     FROM origin_visit
+                       INNER JOIN snapshot ON snapshot=snapshot.id
+                     WHERE origin=origin.id
+                     )
                    AND """
         query += 'url %s %%s '
         if not count:
@@ -779,28 +815,12 @@ class Db(BaseDb):
                            regexp=regexp, with_visit=with_visit, cur=cur)
         return cur.fetchone()[0]
 
-    person_cols = ['fullname', 'name', 'email']
-    person_get_cols = person_cols + ['id']
-
-    def person_get(self, ids, cur=None):
-        """Retrieve the persons identified by the list of ids.
-
-        """
-        cur = self._cursor(cur)
-
-        query = """SELECT %s FROM (VALUES %%s) as t(id)
-                   LEFT JOIN person ON t.id = person.id
-                """ % ','.join('person.' + col for col in self.person_get_cols)
-
-        yield from execute_values_generator(
-            cur, query, ((id,) for id in ids))
-
     release_add_cols = [
         'id', 'target', 'target_type', 'date', 'date_offset',
         'date_neg_utc_offset', 'name', 'comment', 'synthetic',
         'author_fullname', 'author_name', 'author_email',
     ]
-    release_get_cols = release_add_cols + ['author_id']
+    release_get_cols = release_add_cols
 
     def release_get_from_list(self, releases, cur=None):
         cur = self._cursor(cur)
@@ -817,6 +837,10 @@ class Db(BaseDb):
             """ % query_keys,
             ((id,) for id in releases))
 
+    def release_get_random(self, cur=None):
+        return self._get_random_row_from_table(
+            'release', ['id'], 'id', cur)
+
     def origin_metadata_add(self, origin, ts, provider, tool,
                             metadata, cur=None):
         """ Add an origin_metadata for the origin at ts with provider, tool and
@@ -835,19 +859,17 @@ class Db(BaseDb):
         """
         cur = self._cursor(cur)
         insert = """INSERT INTO origin_metadata (origin_id, discovery_date,
-                    provider_id, tool_id, metadata) values (%s, %s, %s, %s, %s)
-                    RETURNING id"""
-        cur.execute(insert, (origin, ts, provider, tool, jsonize(metadata)))
+                    provider_id, tool_id, metadata)
+                    SELECT id, %s, %s, %s, %s FROM origin WHERE url = %s"""
+        cur.execute(insert, (ts, provider, tool, jsonize(metadata), origin))
 
-        return cur.fetchone()[0]
-
-    origin_metadata_get_cols = ['origin_id', 'discovery_date',
+    origin_metadata_get_cols = ['origin_url', 'discovery_date',
                                 'tool_id', 'metadata', 'provider_id',
                                 'provider_name', 'provider_type',
                                 'provider_url']
 
-    def origin_metadata_get_by(self, origin_id, provider_type=None, cur=None):
-        """Retrieve all origin_metadata entries for one origin_id
+    def origin_metadata_get_by(self, origin_url, provider_type=None, cur=None):
+        """Retrieve all origin_metadata entries for one origin_url
 
         """
         cur = self._cursor(cur)
@@ -857,7 +879,7 @@ class Db(BaseDb):
                             %%s)''' % (','.join(
                 self.origin_metadata_get_cols))
 
-            cur.execute(query, (origin_id, ))
+            cur.execute(query, (origin_url, ))
 
         else:
             query = '''SELECT %s
@@ -865,7 +887,7 @@ class Db(BaseDb):
                             %%s, %%s)''' % (','.join(
                 self.origin_metadata_get_cols))
 
-            cur.execute(query, (origin_id, provider_type))
+            cur.execute(query, (origin_url, provider_type))
 
         yield from cur
 
@@ -929,3 +951,19 @@ class Db(BaseDb):
             (provider_name, provider_url))
 
         return cur.fetchone()
+
+    def _get_random_row_from_table(self, table_name, cols, id_col, cur=None):
+        random_sha1 = bytes(random.randint(0, 255) for _ in range(SHA1_SIZE))
+        cur = self._cursor(cur)
+        query = '''
+            (SELECT {cols} FROM {table} WHERE {id_col} >= %s
+             ORDER BY {id_col} LIMIT 1)
+            UNION
+            (SELECT {cols} FROM {table} WHERE {id_col} < %s
+             ORDER BY {id_col} DESC LIMIT 1)
+            LIMIT 1
+            '''.format(cols=', '.join(cols), table=table_name, id_col=id_col)
+        cur.execute(query, (random_sha1, random_sha1))
+        row = cur.fetchone()
+        if row:
+            return row[0]
