@@ -10,16 +10,16 @@ comment on function hash_sha1(text) is 'Compute SHA1 hash as text';
 -- TBLNAME
 --
 -- Args:
---     tblname: name of the table to mimick
+--     tblname: name of the table to mimic
 create or replace function swh_mktemp(tblname regclass)
     returns void
     language plpgsql
 as $$
 begin
     execute format('
-	create temporary table tmp_%1$I
+	create temporary table if not exists tmp_%1$I
 	    (like %1$I including defaults)
-	    on commit drop;
+	    on commit delete rows;
       alter table tmp_%1$I drop column if exists object_id;
 	', tblname);
     return;
@@ -33,23 +33,22 @@ $$;
 -- This is used to create the tmp_directory_entry_<foo> tables.
 --
 -- Args:
---     tblname: name of the table to mimick
+--     tblname: name of the table to mimic
 create or replace function swh_mktemp_dir_entry(tblname regclass)
     returns void
     language plpgsql
 as $$
 begin
     execute format('
-	create temporary table tmp_%1$I
+	create temporary table if not exists tmp_%1$I
 	    (like %1$I including defaults, dir_id sha1_git)
-	    on commit drop;
-        alter table tmp_%1$I drop column id;
+	    on commit delete rows;
+        alter table tmp_%1$I drop column if exists id;
 	', tblname);
     return;
 end
 $$;
 
-
 -- create a temporary table for revisions called tmp_revisions,
 -- mimicking existing table revision, replacing the foreign keys to
 -- people with an email and name field
@@ -58,7 +57,7 @@ create or replace function swh_mktemp_revision()
     returns void
     language sql
 as $$
-    create temporary table tmp_revision (
+    create temporary table if not exists tmp_revision (
         like revision including defaults,
         author_fullname bytea,
         author_name bytea,
@@ -66,13 +65,12 @@ as $$
         committer_fullname bytea,
         committer_name bytea,
         committer_email bytea
-    ) on commit drop;
-    alter table tmp_revision drop column author;
-    alter table tmp_revision drop column committer;
-    alter table tmp_revision drop column object_id;
+    ) on commit delete rows;
+    alter table tmp_revision drop column if exists author;
+    alter table tmp_revision drop column if exists committer;
+    alter table tmp_revision drop column if exists object_id;
 $$;
 
-
 -- create a temporary table for releases called tmp_release,
 -- mimicking existing table release, replacing the foreign keys to
 -- people with an email and name field
@@ -81,14 +79,14 @@ create or replace function swh_mktemp_release()
     returns void
     language sql
 as $$
-    create temporary table tmp_release (
+    create temporary table if not exists tmp_release (
         like release including defaults,
         author_fullname bytea,
         author_name bytea,
         author_email bytea
-    ) on commit drop;
-    alter table tmp_release drop column author;
-    alter table tmp_release drop column object_id;
+    ) on commit delete rows;
+    alter table tmp_release drop column if exists author;
+    alter table tmp_release drop column if exists object_id;
 $$;
 
 -- create a temporary table for the branches of a snapshot
@@ -96,24 +94,24 @@ create or replace function swh_mktemp_snapshot_branch()
     returns void
     language sql
 as $$
-  create temporary table tmp_snapshot_branch (
+  create temporary table if not exists tmp_snapshot_branch (
       name bytea not null,
       target bytea,
       target_type snapshot_target
-  ) on commit drop;
+  ) on commit delete rows;
 $$;
 
+-- create a temporary table for the tools
 create or replace function swh_mktemp_tool()
     returns void
     language sql
 as $$
-    create temporary table tmp_tool (
+    create temporary table if not exists tmp_tool (
       like tool including defaults
-    ) on commit drop;
-    alter table tmp_tool drop column id;
+    ) on commit delete rows;
+    alter table tmp_tool drop column if exists id;
 $$;
 
-
 -- a content signature is a set of cryptographic checksums that we use to
 -- uniquely identify content, for the purpose of verifying if we already have
 -- some content or not during content injection
@@ -693,21 +691,26 @@ $$;
 -- add a new origin_visit for origin origin_id at date.
 --
 -- Returns the new visit id.
-create or replace function swh_origin_visit_add(origin_id bigint, date timestamptz, type text)
+create or replace function swh_origin_visit_add(origin_url text, date timestamptz, type text)
     returns bigint
     language sql
 as $$
-  with last_known_visit as (
+  with origin_id as (
+    select id
+    from origin
+    where url = origin_url
+  ), last_known_visit as (
     select coalesce(max(visit), 0) as visit
     from origin_visit
-    where origin = origin_id
+    where origin = (select id from origin_id)
   )
   insert into origin_visit (origin, date, type, visit, status)
-  values (origin_id, date, type, (select visit from last_known_visit) + 1, 'ongoing')
+  values ((select id from origin_id), date, type,
+          (select visit from last_known_visit) + 1, 'ongoing')
   returning visit;
 $$;
 
-create or replace function swh_snapshot_add(snapshot_id snapshot.id%type)
+create or replace function swh_snapshot_add(snapshot_id sha1_git)
   returns void
   language plpgsql
 as $$
@@ -752,7 +755,7 @@ create type snapshot_result as (
   target_type  snapshot_target
 );
 
-create or replace function swh_snapshot_get_by_id(id snapshot.id%type,
+create or replace function swh_snapshot_get_by_id(id sha1_git,
     branches_from bytea default '', branches_count bigint default null,
     target_types snapshot_target[] default NULL)
   returns setof snapshot_result
@@ -774,7 +777,7 @@ create type snapshot_size as (
   count bigint
 );
 
-create or replace function swh_snapshot_count_branches(id snapshot.id%type)
+create or replace function swh_snapshot_count_branches(id sha1_git)
   returns setof snapshot_size
   language sql
   stable
@@ -828,40 +831,34 @@ as $$
     select dir_id, name from path order by depth desc limit 1;
 $$;
 
--- Find the visit of origin id closest to date visit_date
+-- Find the visit of origin closest to date visit_date
 -- Breaks ties by selecting the largest visit id
-create or replace function swh_visit_find_by_date(origin bigint, visit_date timestamptz default NOW())
-    returns origin_visit
-    language sql
+create or replace function swh_visit_find_by_date(origin_url text, visit_date timestamptz default NOW())
+    returns setof origin_visit
+    language plpgsql
     stable
 as $$
+declare
+  origin_id bigint;
+begin
+  select id into origin_id from origin where url=origin_url;
+  return query
   with closest_two_visits as ((
     select ov, (date - visit_date), visit as interval
     from origin_visit ov
-    where ov.origin = origin
+    where ov.origin = origin_id
           and ov.date >= visit_date
     order by ov.date asc, ov.visit desc
     limit 1
   ) union (
     select ov, (visit_date - date), visit as interval
     from origin_visit ov
-    where ov.origin = origin
+    where ov.origin = origin_id
           and ov.date < visit_date
     order by ov.date desc, ov.visit desc
     limit 1
-  )) select (ov).* from closest_two_visits order by interval, visit limit 1
-$$;
-
--- Find the visit of origin id closest to date visit_date
-create or replace function swh_visit_get(origin bigint)
-    returns origin_visit
-    language sql
-    stable
-as $$
-    select *
-    from origin_visit
-    where origin=origin
-    order by date desc
+  )) select (ov).* from closest_two_visits order by interval, visit limit 1;
+end
 $$;
 
 -- Object listing by object_id
@@ -927,7 +924,7 @@ $$;
 -- origin_metadata functions
 create type origin_metadata_signature as (
     id bigint,
-    origin_id bigint,
+    origin_url text,
     discovery_date timestamptz,
     tool_id bigint,
     metadata jsonb,
@@ -937,32 +934,34 @@ create type origin_metadata_signature as (
     provider_url  text
 );
 create or replace function swh_origin_metadata_get_by_origin(
-       origin integer)
+       origin text)
     returns setof origin_metadata_signature
     language sql
     stable
 as $$
-    select om.id as id, origin_id, discovery_date, tool_id, om.metadata,
+    select om.id as id, o.url as origin_url, discovery_date, tool_id, om.metadata,
            mp.id as provider_id, provider_name, provider_type, provider_url
     from origin_metadata as om
     inner join metadata_provider mp on om.provider_id = mp.id
-    where om.origin_id = origin
+    inner join origin o on om.origin_id = o.id
+    where o.url = origin
     order by discovery_date desc;
 $$;
 
 create or replace function swh_origin_metadata_get_by_provider_type(
-       origin integer,
-       type text)
+       origin_url text,
+       provider_type text)
     returns setof origin_metadata_signature
     language sql
     stable
 as $$
-    select om.id as id, origin_id, discovery_date, tool_id, om.metadata,
+    select om.id as id, o.url as origin_url, discovery_date, tool_id, om.metadata,
            mp.id as provider_id, provider_name, provider_type, provider_url
     from origin_metadata as om
     inner join metadata_provider mp on om.provider_id = mp.id
-    where om.origin_id = origin
-    and mp.provider_type = type
+    inner join origin o on om.origin_id = o.id
+    where o.url = origin_url
+    and mp.provider_type = provider_type
     order by discovery_date desc;
 $$;
 -- end origin_metadata functions
